from datetime import datetime
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import AbstractGetBinDataClass

import pandas as pd
import urllib.request


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the base
    class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        """
        Parse council provided CSVs to get the latest bin collections for address
        """
        # URLs to data sources
        address_csv_url = "https://opendata.leeds.gov.uk/downloads/bins/dm_premises.csv"
        collections_csv_url = "https://opendata.leeds.gov.uk/downloads/bins/dm_jobs.csv"

        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")

        check_postcode(user_postcode)
        check_paon(user_paon)

        data = {"bins": []}  # dictionary for data
        prop_id = 0  # LCC use city wide URPNs in this dataset
        result_row = None  # store the property as a row

        # Get address csv and give it headers (pandas bypasses downloading the file)
        # print("Getting address data...")
        with urllib.request.urlopen(address_csv_url) as response:
            addr = pd.read_csv(
                response,
                names=[
                    "PropertyId",
                    "PropertyName",
                    "PropertyNo",
                    "Street",
                    "Town",
                    "City",
                    "Postcode",
                ],
                sep=",",
            )

        # Get collections csv and give it headers
        # print("Getting collection data...")
        with urllib.request.urlopen(collections_csv_url) as response:
            coll = pd.read_csv(
                response, names=["PropertyId", "BinType", "CollectionDate"], sep=","
            )

        # Find the property id from the address data
        # ("Finding property reference...")
        for row in addr.itertuples():
            if (
                    str(row.Postcode).replace(" ", "").lower()
                    == user_postcode.replace(" ", "").lower()
            ):
                if row.PropertyNo == user_paon:
                    prop_id = row.PropertyId
                    # print(f"Reference: {str(prop_id)}")
                    continue

        # For every match on the property id in the collections data, add the bin type and date to list
        # Note: time is 7am as that's when LCC ask bins to be out by
        job_list = []
        # print(f"Finding collections for property reference: {user_paon} {result_row.Street} "
        #      f"{result_row.Postcode}...")
        for row in coll.itertuples():
            if row.PropertyId == prop_id:
                job_list.append([row.BinType, datetime.strptime(row.CollectionDate, "%d/%m/%y").strftime(date_format)])

        # If jobs exist, sort list by date order. Load list into dictionary to return
        # print("Processing collections...")
        if len(job_list) > 0:
            job_list.sort(key=lambda x: datetime.strptime(x[1], date_format))
            for i in range(len(job_list)):
                job_date = datetime.strptime(job_list[i][1], date_format)
                if datetime.now() < job_date:
                    dict_data = {
                        "type": job_list[i][0],
                        "collectionDate": job_list[i][1],
                    }
                    data["bins"].append(dict_data)
        else:
            print("No bin collections found for property!")

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        for bins in soup.select('div[class*="service-item"]'):
            bin_type = bins.div.h3.text.strip()
            bin_collection = bins.select("div > p")[1]
            if bin_collection:
                dict_data = {
                    "type": bin_type,
                    "collectionDate": datetime.strptime(bin_collection.get_text(strip=True), "%A, %d %B %Y")
                }
                data["bins"].append(dict_data)

        return data



import json
from datetime import datetime

import requests
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        api_url = "https://wasterecyclingapi.eastriding.gov.uk/api/RecyclingData/CollectionsData?"
        api_key = "APIKey=ekBWR8tSiv6qwMo31REEeTZ5FAiMNB"
        api_license = "&Licensee=BinCollectionWebTeam"
        postcode = kwargs.get("postcode")

        # Url is built from multiple parameters
        requests.packages.urllib3.disable_warnings()
        response = requests.get(f"{api_url}{api_key}{api_license}&Postcode={postcode}")
        json_response = json.loads(response.content)["dataReturned"]
        data = {"bins": []}
        collection_tuple = []

        # East riding seems to return the information per postcode, so we only need the first entry for the bin dates
        collection_date = datetime.strptime(
            json_response[0].get("BlueDate"), "%Y-%m-%dT%H:%M:%S"
        ).strftime(date_format)
        collection_tuple.append(("Blue Bin", collection_date))
        collection_date = datetime.strptime(
            json_response[0].get("GreenDate"), "%Y-%m-%dT%H:%M:%S"
        ).strftime(date_format)
        collection_tuple.append(("Green Bin", collection_date))
        collection_date = datetime.strptime(
            json_response[0].get("BrownDate"), "%Y-%m-%dT%H:%M:%S"
        ).strftime(date_format)
        collection_tuple.append(("Brown Bin", collection_date))

        ordered_data = sorted(collection_tuple, key=lambda x: x[1])

        for item in ordered_data:
            dict_data = {
                "type": item[0],
                "collectionDate": item[1],
            }
            data["bins"].append(dict_data)

        return data



import time

import pandas as pd
from selenium import webdriver
from selenium.webdriver.chrome.options import Options
from selenium.webdriver.common.by import By
from selenium.webdriver.common.keys import Keys
from selenium.webdriver.support.ui import Select
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def get_data(self, df) -> dict:
        # Create dictionary of data to be returned
        data = {"bins": []}

        # Output collection data into dictionary
        for i, row in df.iterrows():
            dict_data = {
                "type": row["Collection Name"],
                "collectionDate": row["Next Collection Due"],
            }

            data["bins"].append(dict_data)

        return data

    def parse_data(self, page: str, **kwargs) -> dict:
        page = "https://chiltern.gov.uk/collection-dates"

        # Assign user info
        user_postcode = kwargs.get("postcode")
        user_paon = kwargs.get("paon")

        # Set up Selenium to run 'headless'
        options = webdriver.ChromeOptions()
        options.add_argument("--headless")
        options.add_argument("--no-sandbox")
        options.add_argument("--disable-gpu")
        options.add_argument("--disable-dev-shm-usage")
        options.add_experimental_option("excludeSwitches", ["enable-logging"])

        # Create Selenium webdriver
        driver = webdriver.Chrome(options=options)
        driver.get(page)

        # Enter postcode in text box and wait
        inputElement_pc = driver.find_element(
            By.ID, "COPYOFECHOCOLLECTIONDATES_ADDRESSSELECTION_ADDRESSSELECTIONPOSTCODE"
        )
        inputElement_pc.send_keys(user_postcode)
        inputElement_pc.send_keys(Keys.ENTER)

        time.sleep(4)

        # Select address from dropdown and wait
        inputElement_ad = Select(
            driver.find_element(
                By.ID,
                "COPYOFECHOCOLLECTIONDATES_ADDRESSSELECTION_ADDRESSSELECTIONADDRESS",
            )
        )

        inputElement_ad.select_by_visible_text(user_paon)

        time.sleep(4)

        # Submit address information and wait
        inputElement_bn = driver.find_element(
            By.ID, "COPYOFECHOCOLLECTIONDATES_ADDRESSSELECTION_NAV1_NEXT"
        ).click()

        time.sleep(4)

        # Read next collection information into Pandas
        table = driver.find_element(
            By.ID, "COPYOFECHOCOLLECTIONDATES_PAGE1_DATES2"
        ).get_attribute("outerHTML")
        df = pd.read_html(table, header=[1])
        df = df[0]

        # Parse data into dict
        data = self.get_data(df)

        return data



from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        data = {"bins": []}

        for bins in soup.find_all("div", {"class": lambda L: L and L.startswith("bg-")}):
            for bin in bins.find_all("p"):
                if bin.find("strong"):
                    results = re.search(
                        "Next Collection:(.*?) on ([A-Za-z]+, \\d\\d? [A-Za-z]+ \\d{4})",
                        bin.find("strong").get_text(strip=True))
                    if results:
                        collection_date = datetime.strptime(results.groups()[1], "%A, %d %B %Y")
                        data["bins"].append({
                            "type": bins.h3.get_text(strip=True),
                            "collectionDate": collection_date.strftime(date_format)
                        })

        data["bins"].sort(
            key=lambda x: datetime.strptime(x.get("collectionDate"), date_format)
        )

        return data



# This script pulls (in one hit) the data from Merton Council Bins Data
from bs4 import BeautifulSoup
from uk_bin_collection.uk_bin_collection.common import *
from uk_bin_collection.uk_bin_collection.get_bin_data import \
    AbstractGetBinDataClass


# import the wonderful Beautiful Soup and the URL grabber
class CouncilClass(AbstractGetBinDataClass):
    """
    Concrete classes have to implement all abstract operations of the
    base class. They can also override some operations with a default
    implementation.
    """

    def parse_data(self, page: str, **kwargs) -> dict:
        # Make a BS4 object
        soup = BeautifulSoup(page.text, features="html.parser")
        soup.prettify()

        bin_data_dict = {"bins": []}

        # Search for the specific bin in the table using BS4
        rows = soup.find("table", class_=("collectiondays")).find_all(
            "tr",
            class_=(
                "food-caddy",
                "papercard-wheelie",
                "plastics-boxes",
                "rubbish-wheelie",
                "textiles",
                "batteries",
            ),
        )

        # Loops the Rows
        for row in rows:
            # Get all the cells
            cells = row.find_all("td")
            # First cell is the bin_type
            bin_type = cells[0].get_text().strip()
            # Date is on the second cell, second paragraph, wrapped in p
            collectionDate = cells[1].select("p > b")[2].get_text(strip=True)
            # Make each Bin element in the JSON
            dict_data = {
                "bin_type": bin_type,
                "collectionDate": datetime.strptime(
                    collectionDate, "%d %B %Y"
                ).strftime(date_format),
            }
            # # Add data to the main JSON Wrapper
            bin_data_dict["bins"].append(dict_data)

        return bin_data_dict
